//
//  ORSSerialPortController.m
//  PleaseSpeakSoftly
//
//  Created by Raphael Schaad on 11/27/15.
//  Copyright © 2015 Raphael Schaad. All rights reserved.
//

#import "ORSSerialPortController.h"
#import "ORSSerialPort.h"
#import "ORSSerialPortManager.h"

@interface ORSSerialPortController ()
<ORSSerialPortDelegate>
@property (nonatomic, strong) ORSSerialPort *serialPort;
@end

@implementation ORSSerialPortController

#pragma mark Public Methods

- (void)start
{
    NSArray *ports = [[ORSSerialPortManager sharedSerialPortManager] availablePorts];
    NSLog(@"ports: %@", ports);
    
    self.serialPort = [ORSSerialPort serialPortWithPath:@"/dev/tty.usbserial-FTH7PIOR"];
    self.serialPort.delegate = self;
    
    self.serialPort.baudRate = @9600;
    [self.serialPort open];
    NSLog(@"opened serialPort: %@", self.serialPort);
    
    // To send characters (ensure correct baudRate, e.g. 115200):
//    NSString *string = @"X";
//    NSData *data = [string dataUsingEncoding:NSASCIIStringEncoding];
//    [self.serialPort sendData:data];
//    NSLog(@"sent string: %@ data: %@", string, data);
}

- (void)terminate
{
    [self.serialPort close];
}

#pragma mark Private Methods

- (void)processData:(NSData *)data
{
    // n=100 samples * 2 (hi&lo) + 4 framing
    static const size_t kDataBufferMaxSize = 204;
    static UInt8 dataBuffer[kDataBufferMaxSize];
    static NSUInteger i = 0;
    // Read all bytes for this callback
    for (NSUInteger j = 0; j < data.length; j++) {
        static UInt8 byte;
        [data getBytes:&byte range:NSMakeRange(j, 1)];
        
        //        NSLog(@"byte: %d", byte);
        dataBuffer[i] = byte;
        i++;
        
        // Detect framing across async (but in-order) callback boundaries by reading it into a buffer
        static UInt8 framingBuffer[4];
        if (byte == 1) {
            framingBuffer[0] = 1;
        } else if (byte == 2) {
            framingBuffer[1] = 2;
        } else if (byte == 3) {
            framingBuffer[2] = 3;
        } else if (byte == 4) {
            framingBuffer[3] = 4;
        } else {
            framingBuffer[0] = framingBuffer[1] = framingBuffer[2] = framingBuffer[3] = 0;
        }
        
        if ((framingBuffer[0] == 1 && framingBuffer[1] == 2 && framingBuffer[2] == 3 && framingBuffer[3] == 4) || i >= kDataBufferMaxSize) {
            //            NSLog(@"Framing or buffer overflow");
            
            for (NSUInteger k = 0; k < kDataBufferMaxSize - 4; k += 2) {
                UInt8 hi = dataBuffer[k];
                UInt8 lo = dataBuffer[k + 1];
                //                NSLog(@"hi: %d", hi);
                //                NSLog(@"lo: %d", lo);
                
                CGFloat value = (CGFloat)(hi + lo);
                //                NSLog(@"%f", value);
                static const CGFloat kValueThreshold = 185.0;
                if (value > kValueThreshold) {
                    NSLog(@"%f", value);
                    static dispatch_once_t onceToken;
                    dispatch_once(&onceToken, ^{
                        dispatch_after(dispatch_time(DISPATCH_TIME_NOW, (int64_t)(1.0 * NSEC_PER_SEC)), dispatch_get_main_queue(), ^{
                            dispatch_async(dispatch_get_main_queue(), ^{
                                NSUserNotification *notification = [[NSUserNotification alloc] init];
                                notification.title = @"Please Speak Softly";
                                notification.informativeText = @"Increased noise level in your environment";
                                [[NSUserNotificationCenter defaultUserNotificationCenter] deliverNotification:notification];
                                
                                NSTask *task = [[NSTask alloc] init];
                                task.launchPath = @"/usr/bin/say";
                                task.arguments = @[@"please speak softly"];
                                [task launch];
                            });
                        });
                    });
                }
            }
            
            // Reset
            i = 0;
        }
    }
}

#pragma mark ORSSerialPortDelegate Protocol

- (void)serialPortWasRemovedFromSystem:(ORSSerialPort *)serialPort
{
    NSLog(@"-serialPortWasRemovedFromSystem: %@", serialPort);
}

- (void)serialPort:(ORSSerialPort *)serialPort didReceiveData:(NSData *)data
{
    [self processData:data];
    
//    NSLog(@"-serialPort:DidReceiveData:");
//    NSLog(@"-serialPort:didReceiveData: %@ %@", serialPort, data);
//    NSString *string = [[NSString alloc] initWithData:data encoding:NSASCIIStringEncoding];
//    NSLog(@"string: %@", string);
}

- (void)serialPort:(ORSSerialPort *)serialPort didReceivePacket:(NSData *)packetData matchingDescriptor:(ORSSerialPacketDescriptor *)descriptor
{
    NSLog(@"-serialPort:didReceivePacket:matchingDescriptor: %@ %@ %@", serialPort, packetData, descriptor);
}

- (void)serialPort:(ORSSerialPort *)serialPort didReceiveResponse:(NSData *)responseData toRequest:(ORSSerialRequest *)request
{
    NSLog(@"-serialPort:didReceiveResponse:toRequest: %@ %@ %@", serialPort, responseData, request);
}

- (void)serialPort:(ORSSerialPort *)serialPort requestDidTimeout:(ORSSerialRequest *)request;
{
    NSLog(@"-serialPort:requestDidTimeout: %@ %@", serialPort, request);
}

- (void)serialPort:(ORSSerialPort *)serialPort didEncounterError:(NSError *)error
{
    NSLog(@"-serialPort:didEncounterError: %@ %@", serialPort, error);
}

- (void)serialPortWasOpened:(ORSSerialPort *)serialPort
{
    NSLog(@"-serialPortWasOpened: %@", serialPort);
}

- (void)serialPortWasClosed:(ORSSerialPort *)serialPort
{
    NSLog(@"-serialPortWasClosed: %@", serialPort);
}

@end
